#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stdarg.h>
#include <assert.h>
#include "mad.h"
#include "ibvsmad.h"


void usage(char* argv0)
{
    printf("Usage:\n\t%s -lid <LID> <COMMAND>\n\n\twhere COMMAND may be:\n", argv0);
    printf("\t\t-i2c_get <I2C_ADDR> <MADDR> <SIZE>\n");
    printf("\t\t-i2c_set <I2C_ADDR> <MADDR> <SIZE> <DATA_BYTES>\n");
    printf("\t\t-cr_get  <CR_ADDR> <MADDR>\n");
    printf("\t\t-cr_set  <CR_ADDR> <MADDR> <DATA_WORD>\n");
    exit(1);
}

////////////////////////////////////////////////////////////////////////


#define SETERR(args) do { printf("-E- "); printf args ; printf("\n"); return 1; } while(0)

#define NEXT(s) do {                                                  \
    if (i+1 >= ac)                                                    \
    {                                                                 \
        SETERR(("Missing parameter after \"%s\" switch", s));         \
        exit(1);                                                      \
    } else i++;} while(0)

#define NEXTC(c,param) do {                                                  \
    if (++i >= ac)                                                    \
    {                                                                 \
        SETERR(("Command \"%s\": Missing parameter <%s> ", c , param)); \
        exit(1);                                                      \
    }} while(0)

////////////////////////////////////////////////////////////////////////

int main(int ac, char *av[])
{
    const char* cmdStr = NULL;

    int i;
    char        lid[10], hexa_lid[10];
    u_int32_t   i2c_addr;
    u_int32_t   maddr;
    u_int32_t   data_size;
    u_int32_t   cr_addr;
    u_int32_t   cr_data;
    u_int8_t    data[64];
    mfile*      mf;

    // Command line parameters
    if (ac < 4 || !strncmp(av[0], "-lid", 4))
        usage(av[0]);

    for (i = 1; i < ac; i++)
    {
        if (*av[i] == '-')
        {
            int argLen = strlen(av[i]);

            if (!strncmp(av[i], "-lid", argLen)) {
                NEXT("-lid");
                u_int32_t hexa_lid_num = strtoul(av[i], NULL, 16);
                //convert to decimal string
                sprintf(hexa_lid, "%x", hexa_lid_num);
                sprintf(lid, "%d", hexa_lid_num);
                mf = mopend(lid, MST_TAVOR);
            }
            else if (!strncmp(av[i], "-i2c_get", argLen)) {
                // -i2c_get <I2C_ADDR> <MADDR> <SIZE>
                u_int32_t   args[3];
                char* argNames[3]  = {"i2c_addr", "maddr", "size"};

                for (int argIdx = 0; argIdx < 3 ; argIdx++ ) {
                    char* ep;
                    NEXTC("i2c_get" , argNames[argIdx]);
                    args[argIdx] = strtoul(av[i], &ep, 16);
                    if (*ep) {
                        SETERR(("Bad argument #%d (%s) for i2c_set command (%s). Expecting a hexadecimal integer.", 
                               argIdx,
                               argNames[argIdx],
                               av[i]));
                    }
                }
                i2c_addr  = args[0];
                maddr     = args[1];
                data_size = args[2];
                printf("Destination lid = 0x%s\nI2C slave addr = 0x%x\nMemory addr = 0x%x\nNumber of bytes to read = 0x%x", hexa_lid, i2c_addr, maddr, data_size);
                
                mf->i2c_slave = i2c_addr;
                mread64(mf, maddr, data, data_size);

                printf("Output =");
                for (int dataIdx = 0; dataIdx < data_size ; dataIdx++ ) {
                    printf(" 0x%02x", data[dataIdx]);
                }
                printf("\n");
                
                break;
            }
            else if (!strncmp(av[i], "-i2c_set", argLen)) {
                // i2c_set <I2C_ADDR> <MADDR> <SIZE> <DATA>
                u_int32_t args[4];
                char* argNames[4]  = {"i2c_addr", "maddr", "size", "data"};
        
                for (int argIdx = 0; argIdx < 4 ; argIdx++ ) {
                    char* ep;
                    NEXTC("i2c_set" , argNames[argIdx]);
                    args[argIdx] = strtoul(av[i], &ep, 16);
                    if (*ep) {
                        SETERR(("Bad argument #%d (%s) for i2c_set command (%s). Expecting a hexadecimal integer.", 
                               argIdx,
                               argNames[argIdx],
                               av[i]));
                    }
                }

                i2c_addr  = args[0];
                maddr     = args[1];
                data_size = args[2];

                printf("Destination lid = 0x%s\nI2C slave addr = 0x%x\nMemory addr = 0x%x\nNumber of bytes to write = 0x%x", hexa_lid, i2c_addr, maddr, data_size);
                
                for (int dataIdx = 0; dataIdx < data_size ; dataIdx++ ) {
                    char* ep;
                    if (!av[i])
                        SETERR(("Missing data."));
                    data[dataIdx] = strtoul(av[i], &ep, 16);
                    printf("%02x ", data[dataIdx]);
                    if (*ep) {
                        SETERR(("Bad data for i2c_set command: %s. Expecting a hexadecimal integer.", 
                               av[i]));
                    }
                    i++;
                }
                
                mf->i2c_slave = i2c_addr;
                mwrite64(mf, maddr, data, data_size);

                break;
            } else if (!strncmp(av[i], "-cr_get", argLen)) {
                // -cr_get <CR_ADDR>
                char* ep;

                NEXTC("i2c_get" , "cr_addr");
                cr_addr = strtoul(av[i], &ep, 16);
                if (*ep) {
                    SETERR(("Bad argument cr_addr for cr_get command (%s). Expecting a hexadecimal integer.", 
                           av[i]));
                }

                printf("Destination lid = 0x%s\nCR-Space addr = 0x%x\n", hexa_lid, cr_addr);
                
                data_size = 1;
                mread4(mf, cr_addr, &cr_data);
                printf("Output = 0x%08x\n", cr_data);
                break;
            } else if (!strncmp(av[i], "-cr_set", argLen)) {
                // -cr_set <CR_ADDR> <DATA>
                u_int32_t args[2];
                char* argNames[2]  = {"cr_addr", "data"};
        
                for (int argIdx = 0; argIdx < 2 ; argIdx++ ) {
                    char* ep;
                    NEXTC("cr_set" , argNames[argIdx]);
                    args[argIdx] = strtoul(av[i], &ep, 16);
                    if (*ep) {
                        SETERR(("Bad argument #%d (%s) for i2c_set command (%s). Expecting a hexadecimal integer.", 
                               argIdx,
                               argNames[argIdx],
                               av[i]));
                    }
                }

                cr_addr   = args[0];
                cr_data   = args[1];

                printf("Destination lid = 0x%s\nCR-Space addr = 0x%x\nWord to write = 0x%x\n", hexa_lid, cr_addr, cr_data);

                data_size = 1;
                mwrite4(mf, cr_addr, cr_data);
                break;
            }
        } else {
            SETERR(("Unexpected argument: %s", av[i]));
            break;
        }
    }
    
}
