#!/bin/bash 

# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available at
# <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
# license, available in the LICENSE.TXT file accompanying this
# software.  These details are also available at
# <http://openib.org/license.html>.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
#
# Description: RPM build script
#

MSTHOME=/usr/mst

usage()
{
cat << EOF

    `basename $0` [ -d <install-dir> ]
                        
                The default install-dir is $MSTHOME

EOF
}

while [ $# -gt 0 ]
do
    case $1 in
            -d)
                shift
                MSTHOME=$1
            ;;
            *)
                usage
                exit 1
            ;;
    esac            
    shift
done

BUILD_ENV="env MSTHOME=$MSTHOME"

cd ${0%*/*}

TMPFILE=/tmp/$$.tmp
SOURCE_DIR=`pwd`
KERNEL=`uname -r | sed -e "s/\-/\_/g"`
MST_DIR=${PWD##*/}
NAME=`echo ${PWD##*/} | cut -d \- -f 1,2`
VERSION=`echo ${PWD##*/} | cut -d \- -f 3- | tr -s '-' '_'`
RELVER=`echo ${PWD##*/} | cut -d \- -f 3-`
build_arch=`rpm --showrc | grep -w "build arch" | cut -d : -f 2 | tr -d " "`
spec="mst.spec"

if [ -d /usr/src/redhat ]; then
	RPM_DIR=/usr/src/redhat
elif [ -d /usr/src/packages ]; then
	RPM_DIR=/usr/src/packages
else
	echo Build rpm on this system is not supported...
	exit 1
fi

#Remove tools rpm if it is already installed
rpmstat=`rpm -qa | grep ${NAME}`
if [ "$rpmstat" != "" ]; then
	echo "You have $rpmstat installed, it will be removed now..."
	rpm -e $rpmstat
fi

if [ -f $RPM_DIR/SPECS/$spec ]; then
	rm -f $RPM_DIR/SPECS/$spec 
fi

# Update Package name, version and release in the spec file
sed -e "s/XXXXXX/$VERSION/g" -e "s/YYYYYY/$NAME/g" $spec > $RPM_DIR/SPECS/$spec

cd ..
rm -f /tmp/${NAME}-${RELVER}.tgz
tar czvf /tmp/${NAME}-${RELVER}.tgz $MST_DIR > /dev/null 2>&1
mv -f /tmp/${NAME}-${RELVER}.tgz $RPM_DIR/SOURCES/

if [ -f /etc/SuSE-release ]; then
   if [ `grep VERSION /etc/SuSE-release | cut -d = -f 2 | cut -d . -f 1 | tr -d [:space:]` -ge 9 ]; then
	$BUILD_ENV rpmbuild -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
   else	
	$BUILD_ENV rpm -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
   fi
elif [[ -f /etc/redhat-release || -f /etc/rocks-release ]]; then
	$BUILD_ENV rpmbuild -bb $RPM_DIR/SPECS/$spec # > /dev/null 2>&1
else
        echo Build rpm on this system is not supported...
        exit 1
fi

if [ $? != 0 ]; then
    echo RPM building failed...
    exit 1
fi 

rm -f $RPM_DIR/SOURCES/$MST_DIR.tgz
rm -f $RPM_DIR/SPECS/$spec

if [ ! -f $RPM_DIR/RPMS/$build_arch/${NAME}-${VERSION}-${KERNEL}.${build_arch}.rpm ] ; then
	echo RPM building failed...
	exit 1
else
	echo RPM: $RPM_DIR/RPMS/$build_arch/${NAME}-${VERSION}-${KERNEL}.${build_arch}.rpm 
fi

exit 0

