#!/bin/bash

########################################################################
# 
# Handle TCL 
#
# The idea is to use teh current tclsh and verify its version.
# also get:
# with_tcl = yes/no - if something failed you will get a "no"
# TCL_LIBS to point to the extra libs required
# TCL_INCLUDE to extra include path
# TCL_VERSION to the current version of tcl.
#
# The fucntion will use previously defined globals:
# TclLibBase - points to where we should look for tclConfig.sh
# TCLSH = name of tcl executable
function GetTclEnv () {

    # no need to find tclsh if we know teh lib path some how
    if test -z "$TclLibBase"; then # do we already know?
        # No? Run tclsh and ask it where it lives.
        # Do we know where a tclsh lives?
        if test -z "$TCLSH"; then
            # Try and find tclsh.
            TCLSH=`which tclsh`
            if test -z "$TCLSH"; then
               TCLSH="unknown"
            fi
        fi
    fi

    # get the lib path:
    # But do we know where to get a tclsh?
    if test "${TCLSH}" != "unknown"; then
      echo "INFO: (GetTclEnv) Using tclsh from ${TCLSH}"
      # to avoid .tclshrc issues use from a file... 
      echo "puts \$tcl_library" > /tmp/tcl.conf.$$
      # Make sure nobody defined the TCL_LIBRARY previously
      export TCL_LIBRARY=
      # TCL will use the LD_LIBRARY_PATH to search for the lib so
      # we must make sure it will include the local lib of the tclsh
      if test ! -z "$LD_LIBRARY_PATH"; then
         tclsh_lib=`echo ${TCLSH}|sed 's=/bin/tclsh.*=='`/lib
         export LD_LIBRARY_PATH=$tclsh_lib:$LD_LIBRARY_PATH
      fi
      ExpTclLibBase=`${TCLSH} /tmp/tcl.conf.$$ | sed -e 's,[^/]*$,,'`
      rm /tmp/tcl.conf.$$
    fi
 
    if test -z "$ExpTclLibBase" ; then
        echo "ERROR: (GetTclEnv) Fail to find Tcl library path"
        with_tcl=no
    else
        echo "INFO: (GetTclEnv) Tcl lib path = ${ExpTclLibBase}"
        tclCondifSearchPath="$tclCondifSearchPath ${ExpTclLibBase}"
        tclCondifSearchPath="$tclCondifSearchPath ${ExpTclLibBase}/.."
        tclCondifSearchPath="$tclCondifSearchPath `echo ${TCLSH} | sed s/sh//`"
        tclCondifSearchPath="$tclCondifSearchPath /usr/lib64"
        tclCondifSearchPath="$tclCondifSearchPath /usr/lib"
        tclCondifSearchPath="$tclCondifSearchPath /usr/local/lib"

        # Check a list of places where the tclConfig.sh file might be.
        # prefer lib64 if exists
        for tcldir in $tclCondifSearchPath; do
            if test -f "${tcldir}/tclConfig.sh"; then
                TclLibBase="${tcldir}"
                break
            fi
        done
    fi

    # if failed to find the config file:
    if test -z "${TclLibBase}" ; then
        echo "ERROR: (GetTclEnv) Fail to find tclConfig.sh anywheer under Tcl library path:${TclLibBase}"
        with_tcl=no
    else
        echo "INFO: (GetTclEnv) Found tclConfig.sh in ${TclLibBase}"
    fi

    # abort if any error
    if test "${with_tcl}" != no ; then
        # source the TCL config
        . ${TclLibBase}/tclConfig.sh
        echo "INFO: (GetTclEnv) Found Version: $TCL_VERSION"
    fi

    # find the path to the include file...
    if test "${with_tcl}" != no ; then
      # Now, hunt for the Tcl include files, since we don't strictly
      # know where they are; some folks put them (properly) in the 
      # default include path, or maybe in /usr/local; the *BSD folks
      # put them in other places.
      for tclinclude in "${TclLibBase}/../include" \
                        "${TCL_PREFIX}/include/tcl${TCL_VERSION}" \
                        "${TCL_PREFIX}/include/tcl" \
                        "${TCL_PREFIX}/include" ; do
        if test -r "${tclinclude}/tcl.h" ; then
          echo "INFO: (GetTclEnv) Found tcl.h in ${tclinclude}"
          TCL_INCLUDE="-I${tclinclude}"
          break
        fi
      done
      if test -z "${TCL_INCLUDE}" ; then
         echo "ERROR: (GetTclEnv) can't find Tcl includes; use of Tcl disabled."
         with_tcl=no
      fi
    fi
    
    # Finally, pick up the Tcl configuration if we haven't found an 
    # excuse not to.
    if test "${with_tcl}" != no; then
        # original path we used for tcl build:
        # TCL_LIBS="${TK_LIB_SPEC} ${TK_XLIBSW} ${TCL_LD_SEARCH_FLAGS} ${TCL_LIB_SPEC}"

        # we are using libtool so need to convert to -rpath if at all
        TCL_LIBS1="${TCL_LIB_SPEC} ${TCL_LIBS}"
        # Filter out the ieee - I do not see a shared version for it.
        TCL_LIBS=`echo ${TCL_LIBS1} | sed 's/-lieee//'`
    fi
    
    # finaly cleanup if no tcl
    if test "${with_tcl}" == no ; then
        TCL_LIBS=
        TCL_VERSION=
        TCL_INCLUDE=
    fi
}

GetTclEnv
