/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  Mem.cpp - Memory (incl. CR-space) manipulation class implementation
 *
 *  Version: $Id: Mem.cpp 860 2004-04-18 11:19:27Z alexr $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <map>

#include "Mem.h"

#define MAX_STR 1024
namespace std {}; using namespace std;

map<string, Mem::Cr_record> Mem::_crall;

////////////////////////////////////////////////////////////////////////
Mem::Mem(const char *fwdir) : _fwdir((char *)fwdir),
                              _mem(0x8000), _use(0x8000)
{
    if (_crall.empty())
        read_all();
} // Mem::Mem


////////////////////////////////////////////////////////////////////////
bool Mem::read_all()
{
    char       str[MAX_STR];
    char       fname[100];
    int        line=0;
    FILE       *fp = NULL;

    sprintf(fname, "%s/MT%d.all", _fwdir, 43132);
    if ((fp = fopen(fname, "r")) == NULL)
        return errmsg("Can't open %s - %s", fname, strerror(errno));

    /*
     * Read and parse ".all" file
     */
    //printf("Start parsing \"%s\" file.\n", fname);
    while(fgets(str, MAX_STR, fp))
    {
        char name[200], name1[200], offs_shft[10], siz[10], *p, *endp;
        int  offset, shift=0, size;

        line++;

        // Comments
        if (str[strspn(str, " \t")] == '#')
            continue;

        // Read 3 fields, first:NAME, second:OFFS.SHIFT, third:SIZE
        if (sscanf(str, "%s%s%s", name, offs_shft, siz) != 3)
            return errmsg("Line %d - invalid format", line);
        if ((p = strchr(offs_shft, '.')) != NULL)
        {
            *p = '\0';
            shift=strtol(p+1, &endp, 0);
            if (*endp != '\0')
                return errmsg("Line %d - invalid bit-address format", line);
        }

        // Get offset (in bytes)
        offset = strtol(offs_shft, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid address format", line);
        if (offset%4)
            return errmsg("Invalid offset (0x%x) for field\"%s\"",
                       offset, name);

        // Get size (in bits)
        size = strtol(siz, &endp, 0);
        if (*endp != '\0')
            return errmsg("Line %d - invalid size format", line);

        // Now create proper record and fill cr_space dictionary
        if (size > 32)
        {
            // Special case for long fields
            if (size%32)
                return errmsg("Invalid size (%d) for field \"%s\"",
                           size, name);
            if (shift)
                return errmsg("Invalid shift (%d) for field \"%s\"",
                           shift, name);
            for (int i=0; i<size/32; i++)
            {
                sprintf(name1, "%s:%d", name, i);
                _crall[name1] = Cr_record(offset/4+i, 0, 32);
            }
        }
        else
            _crall[name] = Cr_record(offset/4, shift, size);
    }

    /*
     * Cleanup
     */
    fclose(fp);
    //printf("End of \"%s\" parsing . %d records / %d lines read.\n",
    //       fname, _crall.size(), line);

#if 0
    /*
     * Just print convertion table
     */
    for (map<string, Cr_record>::iterator i=_crall.begin(); i!=_crall.end(); ++i)
    {
        printf("%-47.47s", i->first.c_str());
        printf("0x%04x.%-3d %08x\n",
               i->second.addr, i->second.shift, i->second.mask);
    }
#endif
    return true;
} // Mem::read_all


////////////////////////////////////////////////////////////////////////
bool Mem::Set(const char *name, unsigned int value)
{
    map<string, Cr_record>::iterator it = _crall.find(name);  // CR Names
    if (it == _crall.end())
        return errmsg("Name \"%s\" not found in CR-space.", name);
    Cr_record &crr = it->second;
    unsigned int  new_value = Get(crr.addr, 1);
    new_value &= (~crr.mask);                      // Clean field
    new_value |= (value << crr.shift) & crr.mask;  // Set field
    Set(crr.addr, new_value);
    return true;
} // Mem::Set
