/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) December 2001, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  Mem.h - Memory (incl. CR-space) manipulation class definitions
 *
 *  Version: $Id: Mem.h 866 2004-04-19 09:36:04Z alexr $
 *
 */

#ifndef MEM_H
#define MEM_H

#include <map>
#include <vector>

#include "utils.h"

class Mem : public ErrMsg
{
public:
    // Record per each CR-space entry
    class Cr_record {
    public:
        Cr_record() { addr=0; shift=0; mask=0; }
        Cr_record(int addr_, int shift_, int size)
        {
            addr  = addr_;
            shift = shift_;
            mask = shift ? (1<<shift)-1 : 0;
            mask |= ((shift+size) == 32) ? 0 : ~((1<<(shift+size))-1);
            mask = ~mask;
        }
        int          addr;
        int          shift;
        unsigned int mask;
    };

public:
    Mem(const char *fwdir);

    bool exists(const char *name)
    {
        return _crall.find(name) != _crall.end();
    }

    bool Set(const char *name, unsigned int value);
    void Set(unsigned int addr, unsigned int value)
    {
        if (addr+1 > _mem.size())
        {
            _mem.resize(addr+1, 0);
            _use.resize(addr+1, false);
        }
        //// DEBUG
        //if (_use[addr])
        //    printf("%x USED. WAS %02d, WRITTEN %02d\n", addr, _mem[addr]&0xff, value&0xff);
        _mem[addr] = value;
        _use[addr] = true;
    }
    unsigned int Get(unsigned int addr, int even_uninitilized = 0)
    {
        if (even_uninitilized)
            return _mem[addr];
        if (!_use[addr])
            abort();
        return _mem[addr];
    }
    bool used(const int ind)               { return _use[ind];   }
    unsigned int Size()                    { return _mem.size(); }
    unsigned int operator[](const int ind) { return Get(ind);    }

private:
    bool read_all();

    char                                    *_fwdir;
    std::vector<u_int32_t>                  _mem;    // Memory image
    std::vector<bool>                       _use;    // Memory usage flags
    static std::map<std::string, Cr_record> _crall;  // CR Names
};

#endif
