/*                  - Mellanox Confidential and Proprietary -
 *
 * Copyright (C) August 2003, Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
 *
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 *
 * End of legal section ......................................................
 *
 *  main.cpp - AImage main file
 *
 *  Version: $Id: main.cpp 3004 2006-08-07 13:25:09Z orenk $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <vector>
#include <list>
#include <string>

#include "Board.h"
#include "Eeprom.h"
#include "utils.h"

#define numb_el(x) (sizeof(x)/sizeof(x[0]))
namespace std {}; using namespace std;

const char *g_version   = "1.1.0";
const char *g_build_ver = ""; //BLD_VER_STR;
const char *g_ibadm_ver = ""; //IBADM_VER_STR;
const char *g_cvs_rev   = "$Revision: 3004 $";

////////////////////////////////////////////////////////////////////////
#define USAGE do { usage(av[0]); return 1; } while(0)
void usage(const char *sname)
{
    const char *descr =
        "\n"
        "          InfiniScale EEPROM image compiler.\n"
        "          ----------------------------------\n"
        "\n"
        "Usage:\n"
        "------\n"
        "\n"
        "        %s [SWITCHES]\n"
        "\n"
        "Short switches summary:\n"
        "        -help\n"
        "             Print this mesage\n"
        "        -version\n"
        "             Print version number\n"
        "        -fw          <FW image filename>\n"
        "             Specify input directory with the FW\n"
        "        -board       <board_name>\n"
        "             Specify board name\n"
        "        -conf       <board_name>\n"
        "             Alias for -board flag\n"
	"        -boards\n"
        "             Do nothing but print all possible boards\n"
        "        -wrimage     <output EEPROM image filename>\n"
        "             Write resulting image to file.\n"
        "        -format      <output EEPROM image format>\n"
        "             Output image format. May be IMAGE, DWORDS or BINARY.\n"
        "             Default is IMAGE.\n"
        "        -fwver\n"
        "             Print version of the given fw and exit\n"
        "        -psid        <psid>\n"
	"             PSID is a string of up to 16 bytes, representing the fw configuration (board type).\n"
	"\n";
    printf(descr, sname);
}

////////////////////////////////////////////////////////////////////////
bool swcmp(const char *par, const char *sw)
{
    char *s = (char *)sw;
    char *p = (char *)par;

    while (*s)
        if (*p++ != *s++)
            return *(p-1) == 0 ? true : (*(p-1) == '=');
    return *p == 0 ? true : (*p == '=');
} // swcmp

#define SETERR(args) do { printf args ; return 1; } while(0)
#define SWVAL(s, p) do {                                     \
    char *_eq_p = strchr(av[i], '=');                        \
    if (_eq_p)                                               \
    {                                                        \
        *_eq_p = 0;                                          \
        p = ++_eq_p;                                         \
    }                                                        \
    else if (++i >= ac)                                      \
      SETERR(("Missed parameter after \"%s\" switch\n", s)); \
    else                                                     \
        p = av[i];                                           \
    } while(0)
////////////////////////////////////////////////////////////////////////
int main(int ac, char *av[])
{
    // Switch values and flags
    bool         boards = false;
    bool         fw_ver = false;
    char         *fw_dir = 0;
    char         *EEPROM_image = 0;
    char         *EEPROM_format = "image";
    const char   *board = 0;
    char         *psid  = 0;

    // FW related instances
    Board        brd;

    if (ac < 2)
        USAGE;

    /*
     * Go thru command line options
     */
    for (int i=1; i < ac; i++)
    {
        /*
         * Switches
         * --------
         */
        if (*av[i] == '-')
        {
            char *swval;            // Switch value
            char *orig_sw = av[i];  // Original switch

            // Skip all "-" in prefix
            while (*av[i] == '-')
                ++av[i];

            // FW image
            if (swcmp(av[i], "fw"))
            {
                SWVAL("-fw", swval);
                if (fw_dir)
                    SETERR(("FW directory file may be specified only once.\n"
                           "New FW image file: \"%s\", "
                           "Old FW image file: \"%s\"\n",
			    swval, fw_dir));
                fw_dir = swval;
            }

            // read boadr definition file
            else if (swcmp(av[i], "board"))
            {
                SWVAL("-board", swval);
                board = swval;
            }
            
	    // read boadr definition file (Alias for -board flag)
            else if (swcmp(av[i], "conf"))
            {
                SWVAL("-conf", swval);
                board = swval;
            }

            // print boards list
            else if (swcmp(av[i], "boards"))
            {
                boards = true;
            }

	    // print boards list
            else if (swcmp(av[i], "fwver"))
            {
                fw_ver = true;
            }           

            // write image
            else if (swcmp(av[i], "wrimage"))
            {
                SWVAL("-wrimage", swval);
                if (EEPROM_image)
                    SETERR(("Image output file may be specified only once.\n"
                           "New image file: \"%s\", "
                           "Old image file: \"%s\"\n",
			    swval, EEPROM_image));
                EEPROM_image = swval;
            }
             
	    // psid
            else if (swcmp(av[i], "psid"))
            {
                SWVAL("-psid", swval);
                if (psid)
                    SETERR(("PSID may be specified only once.\n"
                           "New psid: \"%s\", "
                           "Old psid: \"%s\"\n",
			    swval, psid));
                psid = swval;
            }           
	    // Device type (Not needed here - only for unified interface with MIC)
            else if (swcmp(av[i], "dev"))
            {
                SWVAL("-dev", swval);
                string sdev  = swval;
                if      (sdev != "MT43132")
		  SETERR(("Unsupported device type (only MT43132 supported): %s\n", sdev.c_str()));
            }             

            // write image format
            else if (swcmp(av[i], "format"))
            {
                SWVAL("-format", swval);
                if (EEPROM_format)
                    SETERR(("Output image format may be specified only once.\n"
                           "New image format: \"%s\", "
                           "Old image format: \"%s\"\n",
			    swval, EEPROM_format));
                EEPROM_format = swval;

            }

            // version
            else if (swcmp(av[i], "version"))
            {
                printf("aimage %s, IBADM %s, BUILD %s\n", g_version, g_ibadm_ver, g_build_ver);
                return 0;
            }

	    else if (swcmp(av[i], "vv"))
            {
                printf("aimage %s, IBADM %s, BUILD %s, CVS %s\n", g_version, g_ibadm_ver, g_build_ver, g_cvs_rev + 1);
                return 0;
            }

	    // help
            else if (swcmp(av[i], "help"))
            {
                usage(av[0]);
                return 0;
            }

            /*
             * All other switches
             * ------------------
             */
            else
	      SETERR(("Invalid switch \"%s\" is specified.\n", orig_sw));
        }

        else
	  SETERR(("Unexpected parameter \"%s\" is specified.\n", av[i]));
    }

    // Read and parse board defintion file
    string bf_name = string(fw_dir) + "/boards/boards.xml";
    if (!brd.read(bf_name.c_str()))
      SETERR(("\nERROR: %s\n", brd.err()));

    if (fw_ver) {
	if (board == NULL) {
	    board = brd.get_first_board();
	}
    }

    // Print board names if needed
    if (boards)
    {
        brd.print_names();
        return 0;
    }
    if (!board)
        SETERR(("Board name must be specified. You may specify \"-boards\"\n"
		"switch to get list of all known boards.\n"));

    /*
     * Check parameters
     */
    Board::info *pinfo = brd.get(board);
    if (!pinfo)
        SETERR(("Board \"%s\" not found. You may specify \"-boards\"\n"
		"switch to get list of all known boards.\n", board));
    if (!fw_dir)
      SETERR(("FW directory must be specified.\n"));
    if (!EEPROM_image && !fw_ver)
      SETERR(("Output EEPROM image filename must be specified.\n"));
    Board::more_info *minfo = brd.get_more(board);
    if (!minfo)
      SETERR(("INTERNAL ERROR - Board \"%s\" more info not found.\n", board));

    // Go thru board-specific info
    Eeprom eeprom(pinfo, minfo, fw_dir);
    if (eeprom.err())
      SETERR(("ERROR: %s\n", eeprom.err()));
    if (!eeprom.read())
      SETERR(("ERROR: %s\n", eeprom.err()));

    if (fw_ver) {
	printf("%s\n",eeprom.fw_version());
	return 0;
    }

    // Compile EEPROM image
    if (!eeprom.compile(psid))
      SETERR(("ERROR: %s\n", eeprom.err()));

    // Write resulting image
    if (!eeprom.write(board, EEPROM_image))
      SETERR(("ERROR: %s\n", eeprom.err()));

    return 0;
} // main
